// ===================================================================
// (C) Copyright IBM Corp. 2005, 2006 All Rights Reserved.
//                                                                           
// Licensed Materials - Property of IBM
//                                                          
// US Government Users Restricted Rights - Use, duplication or disclosure
// restricted by GSA ADP Schedule Contract with IBM Corp.
//              
// You may copy, modify, and distribute this sample program in any
// form without payment to IBM, for any purpose including developing,
// using, marketing or distributing programs that include or are
// derivative works of the sample program.
//                                                                           
// The sample program is provided to you on an "AS IS" basis, without
// warranty of any kind. IBM HEREBY EXPRESSLY DISCLAIMS ALL WARRANTIES,
// EITHER EXPRESS OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
// Some jurisdictions do not allow for the exclusion or limitation of
// implied warranties, so the above limitations or exclusions may not
// apply to you. IBM shall not be liable for any damages you suffer as
// a result of using, modifying or distributing the sample program or
// its derivatives.
//===================================================================
package com.ibm.wbimonitor.samples.emitterframework.cei;

import javax.naming.InitialContext;
import javax.naming.NamingException;

import org.eclipse.hyades.logging.events.cbe.CommonBaseEvent;

import com.ibm.events.EventsException;
import com.ibm.events.emitter.Emitter;
import com.ibm.events.emitter.EmitterFactory;
import com.ibm.wbimonitor.samples.emitterframework.EventFormatter;
import com.ibm.wbimonitor.samples.emitterframework.EventFormatterFactory;
import com.ibm.wbimonitor.samples.emitterframework.EventHandler;
import com.ibm.wbimonitor.samples.emitterframework.exception.EventFormatFailedException;
import com.ibm.wbimonitor.samples.emitterframework.exception.EventFormatterException;
import com.ibm.wbimonitor.samples.emitterframework.exception.EventHandlerException;

/*
 * This class coordinates the formatting of "native" events to CBEs and the
 * emission of these CBEs into the Common Event Infrastructure.
 */
public abstract class AbstractEventHandler implements EventHandler {

    /*
     * The default JNDI name for the emitter factory to use for CBE events
     * emission. This default JNDI name can be overriden via the
     * "emitterFactoryJNDI" environment entry in the EJB Deployment Descriptor.
     */
    private static final String EMITTER_FACTORY_JNDI_DEFAULT = "iiop://localhost:2809/com/ibm/events/configuration/emitter/Default";

    /*
     * This method handles the formatting of the event from "native" form to a
     * CBE. An appropriate EventFormatter is obtained from the
     * EventFormatterFactory provided by the implementing subclass. This
     * EventFormatter understands the specifics of how to convert the native
     * form of the event into a well-formed CBE expected by the WB Monitor. Upon
     * formatting completion, the newly created CBE is emitted the CEI using the
     * specified EmitterFactory.
     */
    public void handle( java.lang.Object o ) throws EventHandlerException {
        CommonBaseEvent event = null;
        try {
            // Get an appropriate EventFormatter
            EventFormatter formatter = getEventFormatterFactory( ).getEventFormatter( o );

            // Convert the input object to Common Base Event.
            event = formatter.format( o );
        }
        catch( EventFormatFailedException effe ) {
            throw new EventHandlerException( "Could not format to a Common Base Event.", effe );
        }
        catch( EventFormatterException efe ) {
            throw new EventHandlerException( "An appropriate EventFormatter was not found.", efe );
        }

        Emitter emitter = null;
        try {
            // Get the EmitterFactory JNDI name from the environment entry.
            InitialContext ic = new InitialContext( );
            String emitterFactoryJNDI = (String)ic.lookup( "java:comp/env/emitterFactoryJNDI" );

            // If the JNDI name is not explictly specified, use the default JNDI
            // EmitterFactory.
            if( ( emitterFactoryJNDI == null ) || ( emitterFactoryJNDI.length( ) == 0 ) ) {
                emitterFactoryJNDI = EMITTER_FACTORY_JNDI_DEFAULT;
            }

            // Get the EmitterFactory.
            EmitterFactory emitterFactory = (EmitterFactory)ic.lookup( emitterFactoryJNDI );

            // Get the Emitter
            emitter = emitterFactory.getEmitter( );

            // Send the Common Base Event to the CEI Server.
            emitter.sendEvent( event );
        }
        catch( EventsException ee ) {
            if( emitter == null ) {
                throw new EventHandlerException( "Unable to obtain the emitter from the EmitterFactory.", ee );
            }
            else {
                throw new EventHandlerException( "Failure encountered during event emissions.", ee );
            }
        }
        catch( NamingException ne ) {
            throw new EventHandlerException( "Failed to look-up EmitterFactory.", ne );
        }
        finally {
            try {
                if( emitter != null ) {
                    emitter.close( );
                }
            }
            catch( EventsException e ) {
                throw new Error( "", e );
            }
        }
    }

    /*
     * This method is to be implemented by subclasses to return an
     * EventFormatterFactory which has the knowledge of what EventFormatter to
     * use for each of the supported event types.
     */
    protected abstract EventFormatterFactory getEventFormatterFactory( );

}